/**********
This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**********/
// "liveMedia"
// Author Flavio Oliveira
// AAC File Sources
// C++ header

#ifndef _AAC_FILE_SOURCE_HH
#define _AAC_FILE_SOURCE_HH

#ifndef _FRAMED_FILE_SOURCE_HH
#include "FramedFileSource.hh"
#endif

#ifndef _AACINTERNALS_H_
#include "AACInternals.hh"
#endif

#ifndef _BOOLEAN_HH
#include "Boolean.hh"
#endif

class AACFrame; // forward

class AACFileSource: public FramedFileSource {
public:
  static AACFileSource* createNew(UsageEnvironment& env, char const* fileName);

protected:
  AACFileSource(UsageEnvironment& env, FILE* fid);
	// called only by createNew()

  virtual ~AACFileSource();

protected:
  void assignStream(FILE* fid, unsigned filesize);
  Boolean initializeStream();

private:
  // redefined virtual functions:
  virtual void doGetNextFrame();
  virtual char const* MIMEtype() const; 

private:
  virtual Boolean doGetNextFrame1();
  void readFile();
  void getNextFrame(unsigned char* outBuf, unsigned& resultFrameSize);
  Boolean findADTSHeader();
  Boolean findADIFHeader();
  void setSampleFrequency();

private:
  Boolean res;
  hdr fHDR;
  AACFrame* fStreamState;
  Boolean fHaveJustInitialized;
  struct timeval fFirstFramePresentationTime; // set on stream init
  int iSrate;
      
  int n, offset,nread,nData;
  int frameSize, headerSize;
  char *buffer;
};

#endif
