/**********
This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**********/
// "liveMedia"
// Author Flavio Oliveira
// Derived from MP3 Internals
// Implementation

#include <stdio.h>
#include "AACInternals.hh"

#define MAX_LENGTH 32

AACFrame::AACFrame() {

}

AACFrame::~AACFrame() {

}
	
void AACFrame::setup(const unsigned char *data, int nbytes) {
    fBaseBytePtr = data;
    fStartBytePtr = data;
    fEndBytePtr = data + nbytes;
    fCurBitIndex = 0;
    fTotNumBits = 0;
}

void AACFrame::putBits() { 
    unsigned int k = 0; 
    while (fCurBitIndex <= MAX_LENGTH - 8 && fBaseBytePtr != fEndBytePtr) { 
		k = (k << 8) + *(fBaseBytePtr); 
		fBaseBytePtr++; 
		fCurBitIndex += 8; 
    } 
    fTotNumBits |= (k << (MAX_LENGTH - fCurBitIndex)); 
}

Boolean AACFrame::findADTSHeader(const unsigned char *data, int nbytes, int &nBytesRead, hdr *pHdr) {

    Boolean res = true;

	setup(data, nbytes);
	
	if (readBits(12) != 0xfff ) // syncword always: '111111111111'
	    res = False;
	readBits(1);                // ID 0: MPEG-4, ID 1: MPEG-2
	if (readBits(2) != 0)       // Layer always: '00'
	    res = False;
	pHdr->protection_absent = readBits(1);   // Protection absent
	readBits(2);                // Profile
									// 00 (0) Main profile AAC MAIN
									// 01 (1) Low Complexity profile (LC) AAC LC
									// 10 (2) Scalable Sample Rate profile (SSR) AAC SSR
									// 11 (3) (reserved) AAC LTP
	pHdr->sample_frequency_index = readBits(4); // Sampling frequency index
	readBits(1);				// Private bit
	readBits(3);				// Channel configuration
	readBits(1);				// Original/Copy
	readBits(1);				// Home
	readBits(1);                // Copyright identification bit
	readBits(1);				// Copyright identification start
	pHdr->aac_frame_length = readBits(13); // Length of the frame including header (in bytes)
	readBits(11);				// ADTS buffer fullness: 0x7FF indicates VBR
	readBits(2);				// No raw data blocks in frame
		
	if (!pHdr->protection_absent) { 			// only if protection_absent == 0
	    if (pHdr->number_of_raw_data_blocks_in_frame != 1)
		   res = False; // don't support more than one raw data block w/ CRC yet
	    else
	  	   skipBits(16); // skip the CRC	    
	}
	
	nBytesRead = (nBitsRead() + 7) >> 3;
	if (isOutOfBits()) {
		nBytesRead = 0;
		res = False;
	}
	pHdr->header_size = nBytesRead;
		
	return res;
}

Boolean AACFrame::isADIFHeader(const unsigned char *data, int nbytes, int &nBytesRead) {
	Boolean res = True;
	setup(data, nbytes);

	if (readBits(8) != 'A' || readBits(8) != 'D' ||
	    readBits(8) != 'I' || readBits(8) != 'F') {
	    res = False;
	}

	nBytesRead = (nBitsRead() + 7) >> 3;
	if (isOutOfBits()) {
		nBytesRead = 0;
		res = False;
	}

	return res;
}

inline unsigned int AACFrame::peekBits(unsigned int nBits) {
    if (fCurBitIndex < (signed)nBits)
        putBits();
    return (unsigned int)(fTotNumBits >> (MAX_LENGTH - nBits));
}

unsigned int AACFrame::readBits(unsigned int nBits) {
    unsigned int data = peekBits(nBits);
    skipBits(nBits);
    return data;
}

int AACFrame::isOutOfBits() {
    putBits();
    return fCurBitIndex < 0;
}

int AACFrame::nBitsRead() const {
    return 8*(fBaseBytePtr - fStartBytePtr) - fCurBitIndex;
}
